# git-post-receive event.py
#
# Copyright 2010, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#   James Laska <jlaska@redhat.com>

import optparse

name = 'git-post-receive'

def extend_parser(parser):
    '''Extend the given OptionParser object with settings for this event.'''
    parser.set_usage('%%prog %s [options]' % name)
    group = optparse.OptionGroup(parser, '%s options' % name)
    group.add_option('--name', default='',
        help='Git repository name')
    group.add_option('--url', default='',
        help='Git repository URL')
    group.add_option('--branch', default='master',
        help='Git branch to use')
    parser.add_option_group(group)
    return parser

def process_testdata(parser, opts, args, **extra):
    '''Given an optparse.Values object and a list of args (as returned from
    OptionParser.parse_args()), return a dict containing the appropriate key=val
    pairs required by test's control files and test object. The event can also
    call parser.error here if it find out that not all options are correctly
    populated.'''

    if not opts.name:
        parser.error('--name is a mandatory option!')
    if not opts.branch:
        parser.error('--branch is a mandatory option!')
    if not opts.url:
        parser.error('--url is a mandatory option!')

    testdata = {'name': opts.name,
                'branch': opts.branch,
                'baseurl': opts.url,}
    return testdata
