#
# Copyright 2009, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Author: Will Woods <wwoods@redhat.com>

from autotest_lib.client.bin import utils
from autoqa.test import AutoQATest
from autoqa.decorators import ExceptionCatcher

class conflicts(AutoQATest):

    @ExceptionCatcher()
    def initialize(self, config, baseurl, name, parents, **kwargs):
        super(self.__class__, self).initialize(config)
        if name:
            self.detail.id = "%s-%s" % (name, kwargs['arch'])
        else:
            self.detail.id = baseurl
        self.detail.arch = kwargs['arch']

    @ExceptionCatcher()
    def run_once(self, baseurl, parents, name, **kwargs):
        super(self.__class__, self).run_once()

        cmd = './potential_conflict.py --tempcache ' \
              '--repofrompath=target,%s --repoid=target' % baseurl

        for count, parent in enumerate(parents, start=1):
            name = 'parent-%u' % count
            cmd += ' --repofrompath=%s,%s --repoid=%s' % (name, parent, name)

        # TODO set cachedir etc. to test.scratchdir?
        out = utils.system_output(cmd, retain_output=True)
        self.log(out, printout=False)

        file_conflicts = []
        conflicting_pkgs = []
        hitmarker = False
        for line in out.split('\n'):
            if hitmarker:
                line.strip()
                if line.startswith('  /'):
                    file_conflicts.append(line[2:])
                elif line:
                    conflicting_pkgs.append(line)
            elif line.startswith('== File conflicts'):
                hitmarker = True

        file_conflicts = sorted(file_conflicts)
        conflicting_pkgs = sorted(conflicting_pkgs)

        self.write_test_keyval({'file_conflicts_count': len(file_conflicts),
                                'conflicting_pkgs_count': len(conflicting_pkgs)})

        if conflicting_pkgs or file_conflicts:
            self.detail.result = 'FAILED'
        else:
            self.detail.result = 'PASSED'

        self.detail.summary = ('%u packages with file conflicts' %
                               len(conflicting_pkgs))
        summary_detail = '= Conflicting pkgs =\n%s' % '\n'.join(conflicting_pkgs)
        summary_detail += '\n\n= File conflicts =\n%s' % '\n'.join(file_conflicts)
        self.detail.log.summary_detail = summary_detail
