# Copyright 2011, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Author: Liam Li <lili@redhat.com>  Hongqing YANG<hoyang@redhat.com>

'''This module includes all functions related to anaconda test which covers the
   the installation steps and some utility functions'''

import os
import sys
import time
import tempfile
import ConfigParser
import re
import urllib
import subprocess
from autoqa import util
from autoqa.config import Config

logdir = os.getcwd()
scratchdir = Config.paths.tmpdir
keyvals = {}

def command_check(command=[]):
    '''check the command is executable'''
    try:
        assert subprocess.call(command, stdout=subprocess.PIPE) == 0
        return True
    except (IOError, OSError, AssertionError):
        print "Command: %s not found" % command[0]
        return False

def module_check(key=""):
    '''check python module is installed'''
    try:
        assert sys.modules.has_key(key) == True
        return True
    except (IOError, OSError, AssertionError):
        print "Pythond module %s not found" % key
        return False

def get_pid(processname):
    '''get the pid number of a process'''
    pid = None
    pid = subprocess.check_output("ps aux|grep -v grep|grep %s|awk '{print $2}'"
            % processname, shell=True)
    return pid

def kill_process(pid):
    '''kill a process identified by pid'''
    if pid:
        subprocess.call(['kill', '%d' %int(pid)])

def kill_rsyslog_port():
    '''kill the rsyslog port daemon, since it does not stop after rsyslog stops'''
    kill_process(get_pid("rsyslogd_\.\*.pid"))

def configure_rsyslog():
    '''config rsyslog port, configuration file and log directory '''
    args = "python %s -p 6080 -s -o .rsyslogd.conf %s" % ('analog.py', logdir)
    output = subprocess.check_output(args.split(), shell=False)
    process = subprocess.call(output.split(), shell=False)

def check_iso_type(iso):
    '''detect the iso type'''
    iso_type = None
    if iso.find("DVD") != -1:
        iso_type = "DVD"
    elif iso.find("boot") !=-1:
        iso_type = "boot"
    elif iso.find("netinst") !=-1:
        iso_type = "netinst"
    elif iso.find("Live") !=-1:
        iso_type = "Live"
    else:
        print "Invalid ISO type."
    return  iso_type

def log_ksfile(ksfile):
    '''write kick start file to log directory'''
    ks_type = ksfile.split("/", 1)
    if (ks_type[0] == "" or ks_type[0] == "." or ks_type[0] == ".." or
        ks_type[0] == ksfile):
        ksin = open(ksfile)
    elif ks_type[0] == "http:" or ks_type[0] == "ftp:" or ks_type[0] == "nfs:":
        ksin =  urllib.urlopen(ksfile)

    # Save ks file to log dir
    ksout = open(os.path.join(logdir, "ks.cfg"), 'w')
    ksout.write(ksin.read())
    ksout.close()
    ksin.close()

def mediakit_image_test(arch, iso_file, baseurl):
    '''This is a wrapper around the existing installer_image_test.  This method
    will handling mounting and unmounting the provided media.'''

    # mount
    tmp_mnt = tempfile.mkdtemp()
    util.check("mount ISO", subprocess.call(["mount", "-o", "loop,ro",
        iso_file, tmp_mnt], stdout=subprocess.PIPE) == 0)
    timestamp = util.test("images", installer_image_test(arch, baseurl), keyvals)
    keyvals["treeinfo_timestamp"] = timestamp
    keyvals["tree_time"] = int(float(timestamp))
    keyvals["treeinfo_localtime"] = time.ctime(timestamp)

    # umount and cleanup
    util.check("umount ISO", subprocess.call(["umount", tmp_mnt],
        stdout=subprocess.PIPE) == 0)
    if os.path.isdir(tmp_mnt):
        os.rmdir(tmp_mnt)
    return True

def anaconda_disk_switch(guest, iso_path, test_type):
    '''FIXME'''
    if test_type == "cdrom":
    # Swap discs
        util.check("change to %s " % os.path.basename(iso_path),
                   subprocess.call(["virsh", "attach-disk", guest.name,
                        iso_path, "hdc", "--type", "cdrom" ],
                        stdout=subprocess.PIPE) == 0)
        time.sleep(5) # Give a moment for the disc swap to work
        return True
    elif test_type == "dvd" or test_type == "livecd":
        return "DVD install,skip this step"
    else:
        print "Unknown install type:%s" % test_type
        return False

def installer_image_test(arch, baseurl):
    '''http://fedoraproject.org/wiki/QA:Installer_image_presence_test_case'''
    localfile = {}
    c = ConfigParser.SafeConfigParser()
    fetchlist = []
    grabber = util.get_urlgrabber(prefix=baseurl)
    # 1. open .treeinfo
    try:
        treeinfo = util.download('.treeinfo', scratchdir, grabber=grabber)
    except Exception, e:
        print "Failed to download .treeinfo: %s" % str(e)
        return False
    c.read(treeinfo)

    # 2. confirm that the arch listed in the [general] section is correct
    util.check("[general] section in .treeinfo", c.has_section('general'))
    util.check("arch = %s in [general]" % arch, c.get('general','arch') == arch)
    timestamp = float(c.get('general','timestamp'))

    # 3. confirm that the files listed in the [images-$ARCH] section are present
    # and non-empty
    img_s = 'images-%s' % arch
    util.check("[%s] section" % img_s, c.has_section(img_s))
    util.check("kernel", c.has_option(img_s,'kernel'))
    util.check("initrd", c.has_option(img_s,'initrd'))
    for imgtype, relpath in c.items(img_s):
        fetchlist.append(relpath)

    # 4. confirm that a [stage2] section is present and lists a mainimage item
    # if older than f15
    version = float(c.get('general', 'version').split('-', 1)[0])
    if version <= 14:
        util.check("mainimage item in [stage2] section",
              c.has_option('stage2','mainimage'))
        relpath = c.get('stage2','mainimage')
        print "stage2 mainimage: %s" % os.path.basename(relpath)
        fetchlist.append(relpath)

    # 5. confirm that the listed images are present and non-empty
    if 'images/boot.iso' in fetchlist:
        # /images/boot.iso does exist in DVD.iso sometimes. skip boot.iso
        # existence test.
        fetchlist.remove('images/boot.iso')
    for relpath in fetchlist:
        loc = util.download(relpath, scratchdir, grabber=grabber)
        util.check("%s non-empty" % os.path.basename(relpath), util.nonempty(loc))
        localfile[relpath] = loc

    # 6. check that each of the listed files matches the checksum specified in
    # the [checksum] section
    util.check("[checksum] section", c.has_section('checksums'))
    for relpath in localfile:
        sumdata = c.get('checksums', relpath)
        (algo, hexdigest) = sumdata.split(':')
        util.checkhash(localfile[relpath], algo, hexdigest)

    print "sanity check complete."
    print "tree timestamp: %s (local time)" % time.ctime(timestamp)
    return timestamp

def get_initrd(treeinfo):
    '''retrieve initrd'''
    c = ConfigParser.SafeConfigParser()
    c.read(treeinfo)
    arch = c.get('general','arch')
    initrd = c.get('images-%s' % arch,'initrd')
    return initrd

def kernel_boot_test(guest):
    '''http://fedoraproject.org/wiki/QA:Kernel_simple_boot_test_case'''
    # First, save the command we used to create the guest
    # Escape whitespaces in bash-style
    virt_log = open(os.path.join(logdir, "create-guest.sh"), 'w')
    create_command = [(cmd if not re.search('\s', cmd) else '"%s"' % cmd)
                      for cmd in guest.create_command]
    virt_log.write(' '.join(create_command) + "\n")
    virt_log.close()
    print "opening serial console"
    tty = guest.opentty()
    print "  serial console at %s" % tty

    # poll the guest until you see "Welcome To" or it dies.
    print "polling for boot messages"
    bootlog = open(os.path.join(logdir, "boot.log"), 'w')
    data = guest.read_until(lambda data: "\nWelcome to" in data, logfd=bootlog,
                            max_wait=20*60)
    bootlog.close()
    if data:
        version = "[unknown]"
        for line in data.splitlines():
            if line.startswith("[    0.000000] Linux version"):
                version = line.split()[4]
                break
        print "kernel version %s booted, initrd started OK" % version
        guest.kernelver = version
        return version
    print "timeout reached. oh well."
    guest.delete()
    return False

def loader_stage2_test(guest, readtimeout=5*60):
    '''http://fedoraproject.org/wiki/QA:Anaconda_stage2_fetch_test_case'''
    # watch guest until:
    # "Welcome to Fedora for $ARCH"
    print "polling for stage2 startup message"
    condition = lambda data: "Welcome to Fedora" in data
    stage1log = open(os.path.join(logdir, "stage1.log"), 'w')
    data = guest.read_until(condition, logfd=stage1log, readtimeout=readtimeout)
    stage1log.close()

    if data:
        print 'anaconda stage2 has started'
        return 'OK'

    print "timeout reached. oh well."


def watch_for_step(guest, stepname, max_wait=10*60, serial_logfd=None,
                   return_at_start=False, sleep=10):
    '''watch the installation steps in given maximum time'''
    anaconda_log = os.path.join(logdir, "anaconda/anaconda.log")
    dumplog = os.path.join(logdir, "anacdump.txt")
    size = 0 # For tracking changes to anaconda.log
    start = time.time()
    while (time.time() - start) < max_wait:
        if os.path.exists(dumplog): # crash dump = failure
            dump_age = time.time() - os.path.getmtime(dumplog)
            if dump_age < 60:
                print "crash log found - waiting for completion"
            else:
                print "crash log saved to %s. giving up." % dumplog
                return False
        if serial_logfd is not None:
            # check for serial output
            newdata = guest.poll_and_read(0)
            if newdata:
                serial_logfd.write(newdata)
                serial_logfd.flush()
            # check serial output for crash message
            serial_log = open(os.path.join(logdir, "stage2.log"))
            for line in serial_log:
                if line.startswith("install exited abnormally"):
                    print "install exited abnormally. giving up."
                    serial_log.close()
                    return False
            serial_log.close()

        # check for message indicating success
        if os.path.exists(anaconda_log):
            newsize = os.path.getsize(anaconda_log)
            if newsize > size:
                f = open(anaconda_log)
                for line in f:
                    # Sample match
                    # '# 16:48:48,069 INFO anaconda: moving (1) to step postselection'
                    result = re.match("^([\d:,]+) [A-Z]+ .*:.* moving .* to "
                                      "step ([^ ]+)", line.strip())
                    if result is None:
                        continue

                    # keep track of what step we're on
                    (stamp, step) = result.groups()
                    if step not in guest.steplist:
                        print "  anaconda now in step %s" % step
                        guest.steplist.append(step)
                        if return_at_start and step == stepname:
                            return True
                        elif len(guest.steplist) > 1 and \
                            guest.steplist[-2] == stepname:
                            return True
                f.close()
                size = newsize
        # FIXME: file change notifications would be better than active waiting
        time.sleep(sleep)

    print "%u seconds elapsed - giving up." % max_wait

def anaconda_disk_probe(guest):
    '''https://fedoraproject.org/wiki/QA:Anaconda_storage_probe_test_case'''
    guest.serial_logfd = open(os.path.join(logdir, "stage2.log"), 'w')
    # XXX could check for "step installpackages\n" instead
    if watch_for_step(guest, 'storagedone', max_wait=60,
                                            serial_logfd=guest.serial_logfd):
        print "storage setup completed OK"
        return True

def anaconda_find_root_parts(guest):
    guest.serial_logfd = open(os.path.join(logdir, "stage2.log"), 'w')
    # XXX could check for "step installpackages\n" instead
    if watch_for_step(guest, 'findrootparts', max_wait=10*60,
                                            serial_logfd=guest.serial_logfd):
        print "find root parts completed OK"
        return True

def anaconda_finish_package_install(guest):
    '''https://fedoraproject.org/wiki/QA:Anaconda_package_install_test_case'''
    print "Waiting up to 90 minutes for package installation to complete"
    if watch_for_step(guest, 'postinstallconfig', max_wait=90*60,
                      serial_logfd=guest.serial_logfd):
        print "Package installation completed OK"
        # TODO: parse install.log for scriptlet errors
        return True

def anaconda_bootloader_setup(guest):
    '''https://fedoraproject.org/wiki/QA:Anaconda_bootloader_setup_test_case'''
    print "Waiting for bootloader setup to complete"
    if watch_for_step(guest, 'instbootloader', max_wait=5*60,
                      serial_logfd=guest.serial_logfd):
        print "Bootloader setup complete"
        return True

def anaconda_install_finish(guest):
    # XXX unofficial test to see if the install finishes
    # NOTE having 'reboot' in the kickstart makes anaconda skip its normal
    # final step ('complete'). 'dopostaction' is the second-to-last step.
    print "Waiting for step 'dopostaction'"
    if watch_for_step(guest, 'dopostaction', max_wait=5*60,
                      return_at_start=True, serial_logfd=guest.serial_logfd):
        print "Install completed in %u seconds" % int(guest.age())
        return True
