#
# Copyright 2009, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Author: Will Woods <wwoods@redhat.com>

import os
import sys
import pickle
from autoqa import util
from autotest_lib.client.bin import utils
from autoqa.test import AutoQATest
from autoqa.decorators import ExceptionCatcher
import install
import StringIO

class rats_install(AutoQATest):

    @ExceptionCatcher()
    def setup(self, **kwargs):

        retval = utils.system('yum -y install libvirt qemu-kvm python-virtinst '
                              'binutils')
        assert retval == 0

        if utils.system('service libvirtd status', ignore_status=True):
            retval = utils.system('service libvirtd start')
            assert retval == 0

        if utils.system('service rsyslog status', ignore_status=True):
            retval = utils.system('service rsyslog start')
            assert retval == 0


    @ExceptionCatcher()
    def initialize(self, config, baseurl, name, arch, ks, **kwargs):
        super(self.__class__, self).initialize(config)
        self.detail.arch = arch
        # set self.detail.id
        if name:
            self.detail.id = "%s-%s-%s" % (name, arch, ks)
        else:
            self.detail.id = '%s:%s' % (ks, baseurl)


    @ExceptionCatcher()
    def run_once(self, baseurl, name, arch, ks, boot_args="sshd", **kwargs):
        super(self.__class__, self).run_once()

        args = '-s %s -l %s %s' % (self.tmpdir, self.resultsdir, baseurl)
        if boot_args:
            args += ' -x '+ boot_args
        args = args.split()
        out = StringIO.StringIO()

        try:
            # redirect output so that we can log it
            sys.stdout = util.Tee(sys.stdout, out, autoclose=False)
            sys.stderr = util.Tee(sys.stderr, out, autoclose=False)
            # run the test
            success = install.main(args, ks+'.ks')
            # collect results
            if success:
                self.detail.result = 'PASSED'
                self.detail.summary = '%s installed OK' % name
            else:
                self.detail.result = 'FAILED'
                self.detail.summary = '%s failed at init' % name
        except Exception, e:
            self.detail.result = 'FAILED'
            if isinstance(e, util.CheckFailedException):
                self.detail.summary = "%s failed at stage '%s'" % (name, e.message)
            else:
                self.detail.summary = "%s: %s" % (name, e)
        finally:
            # revert back output redirection
            sys.stdout = sys.__stdout__
            sys.stderr = sys.__stderr__

        self.log(out.getvalue(), printout=False)
        if self.detail.result != 'PASSED':
            self.log('Inspect additional detailed logs to examine the issue.')
            self.log('See %s' % self.url['result_dir'])

        # grab extra key = val data
        keyval_file = open(os.path.join(self.resultsdir, 'keyval'))
        keyvals = pickle.load(keyval_file)
        self.write_test_keyval(keyvals)
