#
# Copyright 2009, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Author: Will Woods <wwoods@redhat.com>

import os
import pickle
from autotest_lib.client.bin import utils
from autotest_lib.client.common_lib import error
from autoqa.test import AutoQATest
from autoqa.decorators import ExceptionCatcher

class rats_sanity(AutoQATest):

    @ExceptionCatcher()
    def setup(self, **kwargs):
        pass


    @ExceptionCatcher()
    def initialize(self, config, baseurl, name, parents, **kwargs):
        super(self.__class__, self).initialize(config)
        # set self.detail.id
        if name:
            self.detail.id = "%s-%s" % (name, kwargs['arch'])
        else:
            self.detail.id = baseurl
        # set self.detail.arch
        self.detail.arch = kwargs['arch']


    @ExceptionCatcher()
    def run_once(self, baseurl, parents, name, **kwargs):
        super(self.__class__, self).run_once()

        self.detail.keyvals['baseurl'] = baseurl

        cmd = "./sanity.py -s %s -l %s %s" % (self.tmpdir, self.resultsdir,
                                              baseurl)

        try:
            out = utils.system_output(cmd + " 2>&1", retain_output=True)
            self.log(out, printout=False)
            self.detail.result = 'PASSED'
        except error.CmdError, e:
            self.log(e.result_obj.stdout, printout=False)
            self.detail.result = 'FAILED'

        # grab extra key = val data
        keyval = os.path.join(self.resultsdir, 'keyval')
        if os.path.exists(keyval):
            keyval_file = open(keyval)
            keyvals = pickle.load(keyval_file)
            self.write_test_keyval(keyvals)
            keyval_file.close()
