#
# Copyright 2009, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Author: Will Woods <wwoods@redhat.com>

# take a list of pkg names
# and a repo url

# for each pkg, resolve out its deps
# if anything new shows up - add it to the list of things to be resolved
# wash rinse repeat

import yum
import sys

provides_cache = {}

def resolve_deps(pkg, base):
    deps = []
    for req in pkg.requires:
        if req in provides_cache:
            deps.append(provides_cache[req])
            continue
        try:
            po = base.returnPackageByDep(req)
        # catch TypeError because of bug in yum <= 3.2.25
        except (yum.Errors.YumBaseError, TypeError), e:
            print "ERROR: unresolved dep for %s of pkg %s" % (req[0], pkg.name)
            raise
        provides_cache[req] = po.name
        deps.append(po.name)

    return deps

def main():

    repourl = 'http://download.fedoraproject.org/pub/fedora/linux/development/$basearch/os/'

    start_list = ['@core', '@critical-path-base', '@critical-path-gnome']

    new_items = True
    parsed_items = []
    total_items = []
    total_items.extend(start_list)

    my = yum.YumBase()
    my.repos.disableRepo('*')
    my.add_enable_repo('myrepo', baseurls=[repourl])
    cachedir = yum.misc.getCacheDir()
    suffix = '/$basearch/$releasever'
    my.repos.setCacheDir(cachedir + yum.parser.varReplace(suffix, my.yumvar))

    while new_items:
        new_items = False
        these_items = total_items
        for name in these_items:
            if name.startswith('@'):
                grpid = name[1:]
                group = my.comps.return_group(grpid)
                for pkg in group.mandatory_packages.keys() + group.default_packages.keys():
                    if pkg not in total_items:
                        new_items = True
                        total_items.append(pkg)
                total_items.remove(name)
                continue
            if name not in parsed_items:
                packagelist = my.pkgSack.searchNevra(name=name)
                if packagelist:
                    pkg = packagelist[0]
                else:
                    print "WARNING: unresolved package name: %s" % name
                deps = resolve_deps(pkg, my)
                for n in deps:
                    if n not in total_items:
                        new_items = True
                        total_items.append(n)
                parsed_items.append(name)

    print '\n'.join(sorted(total_items))

if __name__ == '__main__':
    main()
