#
# Copyright 2009, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Author: Will Woods <wwoods@redhat.com>

from autotest_lib.client.bin import utils
from autoqa.test import AutoQATest
from autoqa.decorators import ExceptionCatcher

class repoclosure(AutoQATest):

    @ExceptionCatcher()
    def setup(self, **kwargs):
        utils.system('yum -y install yum-utils')

    @ExceptionCatcher()
    def initialize(self, config, baseurl, name, parents, **kwargs):
        super(self.__class__, self).initialize(config)
        if name:
            self.detail.id = "%s-%s" % (name, kwargs['arch'])
        else:
            self.detail.id = baseurl
        self.detail.arch = kwargs['arch']

    @ExceptionCatcher()
    def run_once(self, baseurl, parents, name, **kwargs):
        super(self.__class__, self).run_once()

        self.detail.keyvals['baseurl'] = baseurl
        # build the repoclosure commandline
        cmd = 'repoclosure --tempcache --newest'
        cmd += ' --repofrompath=target,%s --repoid=target' % baseurl

        for count, parent in enumerate(parents, start=1):
            cmd += ' --repofrompath=parent-%u,%s --repoid=parent-%u' % (count, parent, count)

        # TODO set cachedir etc. to e.g. test.scratchdir?
        out = utils.system_output(cmd, retain_output=True)
        self.log(out, printout=False)

        unresolved_count = out.count('unresolved deps:')
        self.write_test_keyval({'unresolved_dep_count': unresolved_count})

        self.detail.result = "FAILED" if unresolved_count else "PASSED"
        self.detail.summary = '%u packages with unresolved deps' % unresolved_count
