#!/usr/bin/env python
# -*- coding: utf-8 -*-

#   rpmguard_test.py - create a few simple RPMs that can be used to test
#                      rpmguard for correct functionality
#
#   Copyright 2009 Red Hat, Inc.
#       
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as 
#   published by the Free Software Foundation; either version 3 of the 
#   License, or (at your option) any later version.
#       
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#       
#   You should have received a copy of the GNU Affero General Public License
#   along with this program; If not, see <http://www.gnu.org/licenses/>.
#
#   Author: Kamil Paral <kparal@redhat.com>

# Rpmfluff module is needed for this script to work.

import rpmfluff as f

def main():
    
    foo = f.SimpleRpmBuild('foo', '0.1', '1')
    foo2 = f.SimpleRpmBuild('foo', '0.1', '2')
    
    # test requires
    foo.add_requires('fooreq1')
    foo2.add_requires('fooreq2')
    foo.add_requires('fooreq3 = 0.3.4')
    foo2.add_requires('fooreq3 >= 0.2.7')
    
    # test provides
    foo2.add_provides('fooprov1 = 0.1.0')
    
    # test obsoletes
    foo.add_obsoletes('fooobs')
    
    # test conflicts
    foo2.add_conflicts('fooconf >= 1.0')
    
    # test configs
    conf1 = f.SourceFile('conf1','conf1 here')
    conf1c = f.SourceFile('conf1','changed conf1 here')
    foo.add_installed_file('/etc/conf1', conf1, isConfig=True)
    foo2.add_installed_file('/etc/conf1', conf1c, isConfig=True)
    
    conf2 = f.SourceFile('conf2','conf2 here')
    foo2.add_installed_file('/etc/conf2', conf2, isConfig=True)
    
    # test documentation
    foo.add_simple_payload_file()
    
    doc1 = f.SourceFile('doc1', 'doc1')
    foo.add_installed_file('/usr/share/doc/doc1', doc1, isDoc=True)
    foo2.add_installed_file('/usr/share/doc/doc1', doc1, isDoc=True)
    
    # test binaries
    bin1 = f.SourceFile('bin1','binary')
    foo2.add_installed_file('/usr/bin/bin1', bin1, mode=755)
    
    # test mode changes
    file1 = f.SourceFile('justfile1', 'justfile1')
    foo.add_installed_file('/usr/share/justfile1', file1, mode=644)
    foo2.add_installed_file('/usr/share/justfile1', file1, mode=4744)
    
    # make it
    foo.make()
    foo2.make()
    
    return 0

if __name__ == '__main__':
    main()
