#
# Copyright 2009, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Author: Will Woods <wwoods@redhat.com>

from autotest_lib.client.bin import utils
import autoqa.koji_utils, autoqa.util
import re
from autoqa.test import AutoQATest
from autoqa.decorators import ExceptionCatcher

class rpmlint(AutoQATest):

    @ExceptionCatcher()
    def setup(self, **kwargs):
        # Provide a en_US dictionary for the rpmlint enchant test
        utils.system('yum -y install rpmlint hunspell-en')


    @ExceptionCatcher()
    def initialize(self, config, nvr, **kwargs):
        super(self.__class__, self).initialize(config)
        self.detail.id = nvr
        self.koji = autoqa.koji_utils.SimpleKojiClientSession()
        self.koji.check_connection()


    @ExceptionCatcher()
    def run_once(self, nvr, name, kojitag, **kwargs):
        super(self.__class__, self).run_once()

        # add header
        self.log('%s\n%s\n%s' % ('='*40, nvr, '='*40))

        # download packages
        pkgs = self.koji.get_nvr_rpms(nvr, self.tmpdir, debuginfo=False,
                                      src=True)
        # run rpmlint
        cmd = 'rpmlint %s' % ' '.join(pkgs)
        cmd_result = utils.run(cmd+" 2>&1", ignore_status=True,
            stdout_tee=utils.TEE_TO_LOGS, stderr_tee=utils.TEE_TO_LOGS)
        self.log(cmd_result.stdout, printout=False)

        # check command return code
        if cmd_result.exit_status == 0:
            self.detail.update_result('PASSED')
        elif cmd_result.exit_status == 64 or cmd_result.exit_status == 66:
            self.detail.update_result('FAILED')
        else:
            self.log('Unexpected exit status: %s' % cmd_result.exit_status,
                     stderr=True, highlight=True)
            self.detail.update_result('ABORTED')

        # TODO filter known/waived errors/warnings
        # - Need to store the filters somewhere authoritative, get signoff, etc

        # extract rpmlint summary line, e.g.:
        # 3 packages and 0 specfiles checked; 9 errors, 80 warnings.
        if self.detail.result != 'ABORTED':
            match = re.match(r'.*?; ((\d+) errors, (\d+) warnings).',
                             cmd_result.stdout.splitlines()[-1])
            self.detail.summary = match.group(1)
            warn_count = match.group(3)
            # mark as INFO if some warnings present
            if self.detail.result == 'PASSED' and int(warn_count) > 0:
                self.detail.update_result('INFO')

        self.detail.keyvals = {'kojitag':kojitag, 'envr':nvr}

        # submit results, including email opt-in
        self.post_results(optin={'pkg': name, 'kojitag': kojitag})
