# site_autotest.py - site-specific autotest extensions for use in autoqa
#
# this must be copied to autotest/server/, because it is imported by
# autotest/server/server.py
#
# Author: James Laska <jlaska@redhat.com>

import logging
import os
from autotest_lib.server.autotest import BaseAutotest

class SiteAutotest(BaseAutotest):
    def _install(self, host=None, autodir=None, use_autoserv=True,
                 use_packaging=True):

        # NOTE: Due to inconsistent rsync/scp behavior inside autotest, all
        # the transferred directories must end with a slash and must be removed
        # and re-created at the destination machine prior to the transfer.
        # See: http://autotest.kernel.org/ticket/45

        # install AutoQA repository
        logging.debug('Installing AutoQA repository')
        host.run('[ -e /etc/yum.repos.d/fedora-autoqa.repo ] || '
                 'wget -P /etc/yum.repos.d http://repos.fedorapeople.org/repos/'
                 'fedora-qa/autoqa/fedora-autoqa.repo')

        # install required packages
        logging.debug("Installing AutoQA dependencies and other required packages")
        packages = []
        with open('/etc/autoqa/client-deps.conf') as conf:
            packages += [l.strip() for l in conf.readlines()
                     if not l.strip().startswith('#') and l.strip()]
        if packages:
            host.run('yum install -y %s' % ' '.join(map(lambda x: "'%s'" % x, packages)))

        # copy autoqa config files
        logging.debug("Copying AutoQA configuration files")
        srcdir = '/etc/autoqa/'
        destdir = srcdir
        host.run('rm -rf %(dir)s; mkdir -p %(dir)s' % {'dir': destdir})
        host.send_file(srcdir, destdir)

        # copy autoqa library
        logging.debug("Copying AutoQA library")
        import autoqa
        srcdir = os.path.dirname(autoqa.__file__)
        assert srcdir, "Can't find autoqa library on the server"
        srcdir += '/' # trailing slash is important
        site_pkgs = host.run('python -c "from distutils.sysconfig import get_python_lib;'
            'print get_python_lib()"').stdout.strip()
        assert site_pkgs, "Can't find python site-packages directory on the client"
        destdir = site_pkgs + '/autoqa/'
        host.run('rm -rf %(dir)s; mkdir -p %(dir)s' % {'dir': destdir})
        host.send_file(srcdir, destdir)

        logging.info("Installation of AutoQA completed")

        # Finally, continue installation by calling our parent
        super(SiteAutotest, self)._install(host, autodir, use_autoserv, use_packaging)

